use crate::Screen;
use crate::Point;
use std::collections::HashSet;

#[derive(Clone)]
pub struct World {
    pub cleared: HashSet<Point>,
    pub occupied: HashSet<Point>,
}

impl World {
    pub fn new() -> World {
        World {
            cleared: HashSet::new(),
            occupied: HashSet::new(),
        }
    }

    pub fn clear(&mut self, pos: Point) {
        self.cleared.insert(pos);
    }

    pub fn get_valid_movements(&self, pos: &Point, b: &Screen) -> Vec<Point> {
        let moves = b.get_valid_movements(pos);
        moves
            .iter()
            .filter(|p| !self.occupied.contains(p))
            .map(|p| p.clone())
            .collect()
    }
}
