extern crate downcast_rs;
extern crate ncurses;

use ncurses::*;

use std::thread::sleep;
use std::time;

mod lib {
    pub mod point;
    pub mod screen;
    pub mod world;
    pub mod entity;
    pub mod ai;
}

use lib::point::Point;
use lib::screen::{BoardCommand, Screen};
use lib::world::World;
use lib::entity::{Queen, Entities};

fn render(e: &Entities, w: &World, b: &Screen) {
    for c in w.cleared.iter() {
        b.render(c, "x");
    }

    for a in e.data.values() {
        a.render(b);
    }
}

#[cfg(test)]
mod tests {
    // TODO: tests, cleanup code in general
    use super::*;

    #[test]
    fn board() {
        let max_x = 20;
        let max_y = 20;

        let mut board = Screen::new(max_x, max_y);
        let mut world = World::new();

        dbg!(board.is_in_bounds(&Point(0, 0)));
        dbg!(board.get_valid_movements(&Point(0, 0)));
    }
}

fn simulate(e: &mut Entities, w: &mut World, b: &mut Screen) {
    let cmds: Vec<BoardCommand> = e
        .data
        .values_mut()
        .map(|a| a.step(b, &w))
        .collect();

    for cmd in cmds {
        match cmd {
            BoardCommand::Dig(pos) => {
                w.clear(pos);
            }
            BoardCommand::LayEgg(pos, id) => {
                e.add_egg(pos.0, pos.1, id);
            }
            BoardCommand::Hatch(egg_id, queen_id) => {
                let egg = e.data.remove(&egg_id);
                let pos = egg.unwrap().get_position();

                let q = e.data.get_mut(&queen_id).unwrap();
                let queen: &mut Queen = q.downcast_mut::<Queen>().unwrap();
                queen.egg_count -= 1;
                e.add_ant(pos.0, pos.1);
            }
            BoardCommand::Noop => {}
        }
    }
    w.occupied.clear();
    let _ = e.data.values().map(|p| w.occupied.insert(p.get_position()));
}


fn main() {
    initscr();

    /* Invisible cursor. */
    curs_set(CURSOR_VISIBILITY::CURSOR_INVISIBLE);

    let mut max_x = 0;
    let mut max_y = 0;

    getmaxyx(stdscr(), &mut max_y, &mut max_x);

    // TODO: fix renderable to render different colors

    let mut board = Screen::new(max_x, max_y);
    let mut world = World::new();

    let mut entities = Entities::new();
    entities.add_queen(0,0);

    for i in -3..3 {
        for j in -3..3 {
            world.clear(Point(i, j));
        }
    }

    loop {
        // TODO: add way to break out of the loop by hitting a random key
        simulate(&mut entities, &mut world, &mut board);
        render(&entities, &world, &board);
        sleep(time::Duration::from_millis(100));
        refresh();
    }
    endwin();
}
